/*******************************************************************************
+
+  LEDA 5.2  
+
+
+  gw_scc.c
+
+
+  Copyright (c) 1995-2007
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/

// $Revision: 1.2 $  $Date: 2007/02/25 00:47:44 $


#include <LEDA/graph/graph_alg.h>
#include <LEDA/graphics/graphwin.h>

using namespace leda;



void display_scc(GraphWin& gw)
{ graph& G = gw.get_graph();
  node_array<int> comp_num(G);
  STRONG_COMPONENTS(G,comp_num);
  node v;
  gw.set_node_label_type(user_label);
  forall_nodes(v,G) 
  { gw.set_color(v,comp_num[v]%16);
    gw.set_user_label(v,string("%d",comp_num[v]));
  }
}

void new_edge_handler(GraphWin& gw, edge)  { display_scc(gw); }
void del_edge_handler(GraphWin& gw)        { display_scc(gw); }
void new_node_handler(GraphWin& gw, node)  { display_scc(gw); }
void del_node_handler(GraphWin& gw)        { display_scc(gw); }

void about_scc(GraphWin& gw) 
{ window& W = gw.get_window();
 
  panel P;
 
  //P.set_panel_bg_color(win_p->mono() ? white : ivory);
 

  P.text_item("");
  P.text_item("\
SCC illustrates the strongly connected components algorithm.");
P.text_item("");
P.text_item("Please enter a directed graph G.");
P.text_item("");
P.text_item("At any time the strongly connected \
components of G are shown in color and number code, i.e., \
nodes belonging to the same connected component are given the same \
number and the same color.");
  P.text_item("");
  P.text_item("");

  P.button("ok");
  W.disable_panel();
  P.open(W);
  W.enable_panel();
}


 /*
void gw_call(GraphWin& gw, void (*help_info)())
   { help_info(); }
   */

int main()
{
  GraphWin gw("Strongly Connected Components");

  gw.set_init_graph_handler(del_edge_handler);
  gw.set_new_edge_handler(new_edge_handler);
  gw.set_del_edge_handler(del_edge_handler);
  gw.set_new_node_handler(new_node_handler);
  gw.set_del_node_handler(del_node_handler);

  gw.set_directed(true);

  gw.display();
  int h_menu = gw.get_menu("Help");
  gw_add_simple_call(gw,about_scc, "About SCC",h_menu);

  about_scc(gw);

  gw.edit();

  return 0;
}

