/*******************************************************************************
+
+  LEDA 5.2  
+
+
+  gw_shortest_path.c
+
+
+  Copyright (c) 1995-2007
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/

// $Revision: 1.2 $  $Date: 2007/02/25 00:47:44 $


#include <LEDA/graph/graph_alg.h>
#include <LEDA/graphics/graphwin.h>
#include <LEDA/core/map.h>

using namespace leda;


GRAPH<int,int> G;

int min_cost = -4;
int max_cost = 8;

void display_shortest_path(GraphWin& gw)
{ gw.set_flush(false);
  node_array<int> dist(G);
  node_array<edge> pred(G);

  edge e; node v;
  
  SHORTEST_PATH(G,G.first_node(),G.edge_data(),dist,pred);
  node_array<int> type = CHECK_SP(G,G.first_node(),G.edge_data(),dist,pred);

  gw.set_width(G.first_node(),25);
  forall_nodes(v,G) 
  { if (type[v] > 0 )
    { gw.set_color(v,black);
      gw.set_user_label(v,"+");
    }
    if (type[v] == 0 )
    { gw.set_color(v,blue);
      string a = string("%d",dist[v]);
      gw.set_user_label(v,a);
    }
    if (type[v] < 0 )
    { gw.set_color(v,green);
      gw.set_user_label(v,"-");
    }
    
  }

  forall_edges(e,G) 
  { node v = G.target(e);
    if ( e == pred[v] )
    { gw.set_width(e,3); 
      if (type[v] == 0  ) gw.set_color(e,blue);
      if (type[v] == -1 ) gw.set_color(e,green);
      if (type[v] == -2 ) gw.set_color(e,red);
    }
    else
    { gw.set_width(e,1);
      
    }
    gw.set_user_label(e,string("%d",G[e]));
  }

  gw.redraw();

}

void init_handler(GraphWin& gw)
{ edge e;
  forall_edges(e,G) 
  { G[e] = rand_int(min_cost+1,max_cost-1);
    gw.set_slider_value(e,(G[e] - min_cost)/((double) (max_cost - min_cost)));
  }
  display_shortest_path(gw);
}


void new_edge_handler(GraphWin& gw, edge e)  
{ G[e] = rand_int(min_cost+1,max_cost-1);
  gw.set_slider_value(e,(G[e] - min_cost)/((double) (max_cost - min_cost)));
  display_shortest_path(gw); 
}


void del_edge_handler(GraphWin& gw)        { display_shortest_path(gw); }
void new_node_handler(GraphWin& gw,node)   { display_shortest_path(gw); }
void del_node_handler(GraphWin& gw)        { display_shortest_path(gw); }

void edge_slider_handler(GraphWin& gw, edge e, double f)
{ G[e] = int(min_cost + (max_cost - min_cost) * f);
  display_shortest_path(gw); 
}

void end_edge_slider_handler(GraphWin& gw, edge e, double f)
{ G[e] = int(min_cost + (max_cost - min_cost) * f);
  display_shortest_path(gw); 
}


void about_shortpath(GraphWin& gw) 
{ window& W = gw.get_window();
 
  panel P;
 
  //P.set_panel_bg_color(win_p->mono() ? white : ivory);

P.text_item("This demo illustrates the output convention of");
P.text_item("shortest-path algorithms.");
P.text_item(" ");
P.text_item("You will be asked to input two integers min and max.");
P.text_item("Edge costs will be random integers in the range [min,max].");
P.text_item(" ");
P.text_item("You may then edit a graph in the graphics window.");
P.text_item("The first node entered is the source of the shortest-path problem");
P.text_item("The solution is updated after each edit operation.");
P.text_item(" ");
P.text_item("Unreachable nodes are labeled by + infinity.");
P.text_item(" ");
P.text_item("Reachable nodes with finite distance are labeled with their distance.");
P.text_item(" ");
P.text_item("Reachable nodes for which there are paths with");
P.text_item("arbitrary negative length are labeled with -infinity.");
P.text_item(" ");
P.text_item("For each reachable node one of the incoming edges is highlighted.");
P.text_item("For each node with finite distance this is the");
P.text_item("last edge on a shortest path to the node.");
P.text_item("For the nodes with distance minus infinity these edges form");
P.text_item("negative cycles plus paths emanating from these cycles."); 

  P.text_item("");


  P.button("OK");
  W.disable_panel();
  P.open(W);
  W.enable_panel();
}

int main()
{ 
  GraphWin gw(G,"Shortest Paths");

  gw.display();

  int h_menu = gw.get_menu("Help");
  gw_add_simple_call(gw,about_shortpath, "About Shortest Paths",h_menu);

  about_shortpath(gw);

  gw.set_action(A_LEFT|A_IMMEDIATE|A_EDGE,0);
  gw.set_action(A_LEFT|A_DRAG|A_EDGE,0); // prevent adding edge bends
  gw.set_action(A_LEFT|A_EDGE,0);


  gw.set_init_graph_handler(del_edge_handler);
  gw.set_new_edge_handler(new_edge_handler);
  gw.set_del_edge_handler(del_edge_handler);
  gw.set_new_node_handler(new_node_handler);
  gw.set_del_node_handler(del_node_handler);

  gw.set_directed(true);
  gw.set_node_shape(circle_node);
  gw.set_node_label_type(user_label);
  gw.set_node_width(20);
#ifdef BOOK
  gw.set_node_color(white);
#endif
  gw.set_edge_slider_handler(edge_slider_handler);
  gw.set_end_edge_slider_handler(end_edge_slider_handler);


  gw.edit();

  return 0;
}

