/*******************************************************************************
+
+  LEDA 5.2  
+
+
+  gw_menu.c
+
+
+  Copyright (c) 1995-2007
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/

// $Revision: 1.2 $  $Date: 2007/02/25 00:47:47 $


#include <LEDA/graphics/graphwin.h>
#include <LEDA/graph/graph_alg.h>
#include <LEDA/graph/graph_misc.h>

using namespace leda;


// a caller for component algorithms

void call_comp(GraphWin& gw, 
               int (*comp)(const graph& G, 
                           node_array<int>& compnum) )
{ graph& G = gw.get_graph();
  node_array<int> compnum(G);
  comp(G,compnum);
  node v;
  forall_nodes(v,G) 
  { int i = compnum[v];
    gw.set_label(v,string("%d",i));
    gw.set_color(v,(color)(i%16));
   }
  if (gw.get_flush() == false) gw.redraw();
}



void dfs_num(GraphWin& gw) 
{ graph& G = gw.get_graph();
  node_array<int> dfsnum(G);
  node_array<int> compnum(G);

  DFS_NUM(G,dfsnum,compnum);

  node v;
  forall_nodes(v,G) 
      gw.set_label(v,string("%d|%d",dfsnum[v],compnum[v]));
  
  if (gw.get_flush() == false) gw.redraw();
}

void span_tree(GraphWin& gw) 
{ graph& G = gw.get_graph();
  list<edge> L = SPANNING_TREE(G);
  gw.set_color(L,red);
  gw.set_width(L,2);
  if (gw.get_flush() == false) gw.redraw();
}


int main() 
{
  GraphWin gw;

  // we delete some of the standard menus
  gw.set_default_menu(M_COMPLETE 
                      & ~M_LAYOUT &  ~M_OPTIONS & ~M_HELP & ~M_WINDOW);

  // add two simple function calls
  gw.add_simple_call(dfs_num,   "dfsnum");
  gw.add_simple_call(span_tree, "spanning");

  // a member call
  gw.add_member_call(&GraphWin::reset,"reset");

  // and a menu with three non-simple functions using 
  // a common call function

  int menu1 = gw.add_menu("components");

  gw_add_call(gw,COMPONENTS,       call_comp, 
                                   "simply connected",  menu1);
  gw_add_call(gw,COMPONENTS1,      call_comp, 
                                   "simply connected1", menu1);
  gw_add_call(gw,STRONG_COMPONENTS,call_comp, 
                                   "strongly connected",menu1);

  gw.display();
  gw.edit();

  return 0;
}
