/*******************************************************************************
+
+  LEDA 5.2  
+
+
+  draw.c
+
+
+  Copyright (c) 1995-2007
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/

// $Revision: 1.2 $  $Date: 2007/02/25 00:47:51 $

#include <LEDA/geo/plane.h>
#include <LEDA/graphics/window.h>
#include <LEDA/core/map.h>

#if defined(LEDA_NAMESPACE)
using namespace leda;
#endif

enum { POINT, SEGMENT, RAY, LINE, CIRCLE, RECT, POLY};



class geo_object_base {
public:
virtual void draw(window& W) = 0;
virtual ~geo_object_base() {}
};


template <class T>
class geo_object : public geo_object_base {

T obj;
color col;
color fcol;

public:

geo_object(const T& x, color c, color fc) : obj(x), col(c), fcol(fc) {}

void draw(window& W)
{ W.set_fg_color(col);
  W.set_fill_color(fcol);
  W << obj;
}

~geo_object() {}

};



class geo_scene {

  list<geo_object_base*> L;
  window& W;

public:

  window& get_window() const { return W; }

  void insert(geo_object_base* p) { L.append(p); }
  void draw()  { geo_object_base* p; forall(p,L) p->draw(W); }
  void clear() { while (!L.empty()) delete L.pop(); }
  geo_scene(window& win) : W(win) {} 

};


template <class T>
inline void add(geo_scene& S)     
{ window &W = S.get_window();
  T x; W >> x;
  geo_object<T>* p = new geo_object<T>(x,W.get_fg_color(),W.get_fill_color());
  p->draw(W);
  S.insert(p); 
}


static window W(500,570);
static geo_scene sc(W);


void redraw(window* wp) 
{ wp->start_buffering();
  wp->clear();
  sc.draw();
  wp->flush_buffer();
  wp->stop_buffering();
}


int main()
{
  color fg_col = red;
  color fill_col = yellow;
  color bg_col = white;

  point_style pstyle = cross_point;
  line_style  lstyle = solid;

  int   shape = CIRCLE;

//int   x_origin = 0;
//int   y_origin = 0;

  list<string> shapes;
  shapes.append("point");
  shapes.append("seg");
  shapes.append("ray");
  shapes.append("line");
  shapes.append("circ");
  shapes.append("rect");
  shapes.append("poly"); 

  W.choice_item("shape",shape,shapes);
  W.color_item ("line color",fg_col);
  W.color_item ("fill color",fill_col);
//W.color_item ("background",bg_col);
  W.pstyle_item("pstyle",pstyle);
  W.lstyle_item("lstyle",lstyle);
  W.text_item("");

  W.button("zoom +",100);
  W.button("zoom -",101);
  W.button("clear",1);
  W.button("quit",2);

  W.init(0,500,0);
  W.set_clear_on_resize(false);
  //W.set_redraw(redraw);
  W.display(window::center,window::center);


  for(;;)
  { 
    point p;
    int k = W.read_mouse(p);

    if (k==2) break;

    W.set_fg_color(fg_col);
    W.set_fill_color(fill_col);
    W.set_bg_color(bg_col);
    W.set_line_style(lstyle);
    W.set_point_style(pstyle);


    switch (k) {

     case   1: W.clear();
               sc.clear();
               continue;
     case 100: W.zoom(2.0);
               continue;
     case 101: W.zoom(0.5);
               continue;
    }

    bool dblclick,drag;
    W.read_mouse(p,300,300,dblclick,drag);

    if (dblclick)
    { point a,b;
      W.read_zoom_rect(p,a,b);
      W.zoom_area(a,b);
      continue;
     }

    if (drag)
    { W.scroll_window(p);
      continue;
     }

    W.set_point_buffer(p);
  


    switch (shape) {

        case POINT:   //sc.add<point>();
                      add<point>(sc);
                      break;
  
        case SEGMENT: //sc.add<segment>();
                      add<segment>(sc);
                      break;

        case RAY:     //sc.add<ray>();
                      add<ray>(sc);
                      break;

        case LINE:    //sc.add<line>();
                      add<line>(sc);
                      break;
  
        case CIRCLE:  //sc.add<circle>();
                      add<circle>(sc);
                      break;

        case RECT:    //sc.add<rectangle>();
                      add<rectangle>(sc);
                      break;
  
        case POLY:    //sc.add<polygon>();
                      add<polygon>(sc);
                      break;
       }

  }

  return 0;
}
