/*******************************************************************************
+
+  LEDA 5.2  
+
+
+  gw_sort_edges_demo.c
+
+
+  Copyright (c) 1995-2007
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/

// $Revision: 1.2 $  $Date: 2007/02/25 00:47:52 $


#include <LEDA/graphics/graphwin.h>
#include <LEDA/graph/graph_alg.h>

using namespace leda;

#if defined(LEDA_STD_IO_HEADERS)
using std::cout;
using std::endl;
#endif


graph G;

void sort(GraphWin& gw)
{ 
  node_array<double> x(G);
  node_array<double> y(G);

  node v;
  forall_nodes(v,G)
  { point p = gw.get_position(v);
      x[v] = p.xcoord();
      y[v] = p.ycoord();
  }

  SORT_EDGES(G,x,y);

  cout << "\n\nThe adjacency lists are:\n";
  G.print();
}

void init_handler(GraphWin& gw)
{ list<edge> L;
  G.make_map(L);
  sort(gw);
}


void new_edge_handler(GraphWin& gw, edge e)  
{ G.set_reversal(e,gw.new_edge(G.target(e),G.source(e)));
  sort(gw);
}


bool del_edge_handler(GraphWin& gw, edge e) 
{ gw.del_edge(G.reversal(e)); return true; }
void new_node_handler(GraphWin& gw,node)   {}
void del_node_handler(GraphWin& gw)        {}


int main()
{ GraphWin gw(G,"Plane Map from Geometry");
 
  gw.set_init_graph_handler(init_handler);
  gw.set_new_edge_handler(new_edge_handler);
  gw.set_del_edge_handler(del_edge_handler);
  gw.set_new_node_handler(new_node_handler);
  gw.set_del_node_handler(del_node_handler);

  gw.set_directed(true);

  gw.display();
  gw.edit();

  return 0;
}


