/*******************************************************************************
+
+  LEDA 5.2  
+
+
+  graphwin.c
+
+
+  Copyright (c) 1995-2007
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/

// $Revision: 1.2 $  $Date: 2007/02/25 00:47:55 $

#include <LEDA/graphics/graphwin.h>
#include <LEDA/system/file.h>

#if defined(LEDA_NAMESPACE)
using namespace leda;
#endif


#if defined(WINMAIN)

int main() {
  int    argc;
  char** argv;
  get_command_line(argc,argv);

#else

int main(int argc, char** argv) {

#endif

  string frame_label("Leda Graph Editor (GraphWin %.1f)",
                                              GraphWin::version());

  GraphWin gw(frame_label);

  window& W = gw.get_window();


  gw.display(400,window::center);

  W.set_icon_label("GraphWin");

  W.set_cursor(XC_watch);

  string home_dir = get_home_directory();

  home_dir = home_dir.replace_all("\\","/");

  string rc_file = home_dir + "/.graphwinrc";

  if (is_file(rc_file))
   { gw.message("Loading Options from\\blue " + rc_file);
     leda_wait(1.2);
     gw.read_defaults();
    }
  else
   { gw.message("No\\blue " + rc_file + "\\black ~(using built-in defaults).");

     leda_wait(1.8);
    }

  gw.message("");


  if (argc > 1) 
  { string fname = argv[1];
    string gname;
    if (fname.tail(4) == ".gml")
      { gname = fname.del(".gml");
        int err = gw.read_gml(fname);
        if (err) gw.acknowledge("Error Reading GML-File", fname);
       }
    else
      if (fname.tail(7) == ".dimacs")
        { gname = fname.del(".dimacs");
          int err = gw.read_dimacs(fname);
          if (err) gw.acknowledge("Error Reading Dimacs File", fname);
         }
      else
        { gname = fname.del(".gw");
          int err = gw.read_gw(fname);
          if (err && err < 5) gw.acknowledge("Error Reading GW-File", fname);
         }
    gw.set_graphname(gname);
   }


  int but = 0;

  //bool save_defaults = false;

  int def_choice = 0;

  W.set_cursor();

  while (but == 0)
  {
    gw.edit();

    panel P;
    P.text_item("");
    P.text_item("\\bf\\blue Exit GraphWin");
    P.choice_item("Options in ~/.graphwinrc",def_choice,"keep","write","reset");

    if (gw.unsaved_changes())
    { P.text_item("\\bf Graph not written since last change !\\n");
      P.button("save gw",3);
      P.button("save gml",4);
     }

    P.button("cancel",  0);
    P.button("exit",1);
    P.buttons_per_line(4);

    but = gw.open_panel(P);

    if (but == 3) gw_save_handler(gw);
    if (but == 4) gw_save_gml_handler(gw);

    if (but != 1 && gw.unsaved_changes()) but = 0;
   }

   if (def_choice != 0) 
   { if (def_choice == 2) gw.reset_defaults();
     gw.save_defaults();
    }

  return 0;
}

