/*******************************************************************************
+
+  LEDA 4.4.1  
+
+
+  flow_test.c
+
+
+  Copyright (c) 1995-2003
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/

// $Revision: 1.1.1.1 $  $Date: 2005/09/12 10:38:19 $


#include <LEDA/graph/graph.h>
#include <LEDA/graph/graph_alg.h>
#include <LEDA/graph/dimacs.h>
#include <LEDA/internal/std/fstream.h>

#if defined(LEDA_NAMESPACE)
using namespace leda;
#endif

#if defined(LEDA_STD_IO_HEADERS)
using std::ofstream;
using std::cout;
using std::flush;
using std::endl;
#endif

int main() 
{
  int node_slots = 2; //read_int("# node slots = ");
  int edge_slots = 1; //read_int("# egde slots = ");

  GRAPH<int,int> G(node_slots,edge_slots);
  node s,t;

  int n = read_int("|V| = ");
  int m = read_int("|E| = ");

  edge_array<int>& cap = G.edge_data();

  for(int j=0; j<4; j++)
  { cout << endl;
    switch (j)
    { 
      case 0: cout << "Random: Random Networks     " << flush;
              max_flow_gen_rand(G,s,t,n,m);
              break;

      case 1: cout << "CG1: Cherkassky/Goldberg 1  " << flush;
              max_flow_gen_CG1(G,s,t,n/5);
              break;

      case 2: cout << "CG2: Cherkassky/Goldberg 2  " << flush;
              max_flow_gen_CG2(G,s,t,n);
              break;

      case 3: cout << "AMO: Ahuja/Magnanti/Orlin   " << flush;
              max_flow_gen_AMO(G,s,t,n);
              break;
     }

    edge_array<int> flow;
    flow.use_edge_data(G,0);
  
    int n = G.number_of_nodes();
    int m = G.number_of_edges();
    
    
    cout << "  (" << n << " nodes  " << m << " edges)" << endl;
    cout << endl;
//  cout << string("Saving problem to flow%d.dim (DIMACS format) ... ",j);
//  cout << flush;
    ofstream out(string("flow%d.dim",j));
    Write_Dimacs_MF(out,G,s,t,cap);
//  cout << endl;
    
    int pushes,relabels,updates, /*gaps,*/ gap_nodes,inspects;
    
    float T;
    int f;
    
    /*  
    cout << "MAX_FLOWS   " << flush;
    T  = used_time();
    f = MAX_FLOWS(G,s,t,cap,flow); //,pushes,relabels,updates,gaps,gap_nodes) ;
    cout << string("time: %6.2f sec  f = %d",used_time(T),f) << endl;

    cout << "            " << flush;
    //cout << string("pushes:%d  relabels:%d  updates:%d  gaps:%d  gap nodes:%d",
//                    pushes, relabels, updates, gaps, gap_nodes) << endl;
    cout << endl;
    CHECK_MAX_FLOW(G,s,t,cap,flow);
    
    */
  
    cout << "MAX_FLOW    " << flush;
    T = used_time();
    f = MAX_FLOW(G,s,t,cap,flow,pushes,inspects,relabels,updates,gap_nodes,5) ;
    cout << string("time: %6.2f sec  f = %d",used_time(T),f) << endl;

    cout << "            " << flush;
    /*
cout << string("pushes:%d  relabels:%d  updates:%d  insp:%d  gap nodes: %d",
                    pushes, relabels, updates, inspects, gap_nodes) << endl;
    cout << endl;

  //CHECK_MAX_FLOW(G,s,t,cap,flow);
  //cout << string("check: %6.2f sec",used_time(T)) << endl;
  */
 }
    
 return 0;
}


