/*******************************************************************************
+
+  LEDA 5.2  
+
+
+  mfs0.c
+
+
+  Copyright (c) 1995-2007
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/

// $Revision: 1.2 $  $Date: 2007/02/25 01:11:32 $

#define LEDA_CHECKING_OFF

#include <LEDA/graph/graph.h>
#include <LEDA/graph/dimacs_graph.h>
#include <LEDA/graph/templates/max_flow4.h>
#include <LEDA/system/assert.h>

using namespace leda;

#if defined(LEDA_STD_IO_HEADERS)
using std::cout;
using std::cin;
using std::flush;
using std::endl;
#endif



int main(int argc, char** argv)
{
  GRAPH<int,int> G(3,1);
  node s=0;
  node t=0;

  edge_array<int>&  cap = G.edge_data();
  //edge_array<int>  cap;
  //assert(cap.use_edge_data(G));

  edge_array<int>  flow;
  assert(flow.use_edge_data(G));


  cout << endl;
  cout << "reading DIMACS maxflow problem from std input ... ";
  cout << flush;

  read_dimacs_mf(cin,(graph&)G,s,t,cap);
  cout << endl;

/*
  list<edge> E = G.all_edges();
  E.reverse_items();
  G.sort_edges(E);
*/

  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  cout << string("%6d nodes",n) << endl;
  cout << string("%6d edges",m) << endl;
  cout << endl;


  cout << "MAX_FLOW    " << flush;

  max_flow<int,graph>  mf;


  if (argc >= 2)  
     mf.set_heuristic(atof(argv[1]));

  mf((graph&)G,s,t,cap,flow);


  cout << endl;
  cout << "     ";
  cout << string("time: %6.2f sec  f = %d", mf.cpu_time(), mf.flow()) << endl;

  mf.statistics(cout);

  mf.check();

  return 0;
}




