/*******************************************************************************
+
+  LEDA 5.2  
+
+
+  mf_preflow_push.cpp
+
+
+  Copyright (c) 1995-2007
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/

// $Revision: 1.2 $  $Date: 2007/02/25 01:11:34 $

#if defined(_MSC_VER) && _MSC_VER < 1300 || defined(__BORLANDC__)
#error This program cannot be compiled with this compiler!
#endif

#define LEDA_CHECKING_OFF

#include <LEDA/graph/graph.h>
#include <LEDA/graph/static_graph.h>
#include <LEDA/graph/dimacs_graph.h>
#include <LEDA/system/assert.h>

#include <LEDA/graph/node_slot.h>
#include <LEDA/graph/edge_slot.h>

#include <LEDA/graph/templates/max_flow.h>


using namespace leda;

#if defined(LEDA_STD_IO_HEADERS)
using std::cout;
using std::cin;
using std::cerr;
using std::flush;
using std::endl;
using std::ofstream;
#endif

int main()
{
/*
  const int ns = max_flow_alg::node_slots;
  const int es = max_flow_alg::edge_slots;
*/

  typedef static_graph<opposite_graph,data_slots<3>,data_slots<2> > st_graph;
  typedef st_graph::node node;
  typedef st_graph::edge edge;


  st_graph G;
  node s=0;
  node t=0;

  edge_slot<int,st_graph,0>   cap(G);
  edge_slot<int,st_graph,1>  flow(G);

  typedef node_slot<node,st_graph,0> succ_array;
  typedef node_slot<int, st_graph,1> excess_array;
  typedef node_slot<int, st_graph,2> dist_array;


/*
  cout << endl;
  cout << "reading DIMACS maxflow problem from std input ... ";
  cout << flush;
*/

  dimacs_mf D(cin);
  D.translate(G,s,t,cap);

/*
  cout << endl;

  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  cout << string("%6d nodes",n) << endl;
  cout << string("%6d edges",m) << endl;
  cout << endl;


  cout << "MAX_FLOW    " << flush;
*/

  max_flow<int,st_graph,succ_array,excess_array,dist_array> mf;

  mf.run(G,s,t,cap,flow);

  cout << string("time\t %.2f", mf.cpu_time()) << endl;  
  cout << string("result\t f = %d", mf.flow()) << endl;
  cout << endl;

  mf.statistics(cout);

  //mf.print(G,s,t,cap,flow);

  string msg;
  if (!mf.check(G,s,t,cap,flow, msg))
  { cerr << msg << endl;
    ofstream os("err.dimacs");
    write_dimacs_mf(os,G,s,t,cap);
    return 1;
   }

  return 0;
}




